#include <pch.h>

SDL_GPUGraphicsPipeline* GraphicsPipelineCreate(SDL_GPUDevice* device, SDL_Window* window,
    SDL_GPUShader* vertex_shader, SDL_GPUShader* fragment_shader) {
    SDL_assert(device != NULL);
    SDL_assert(window != NULL);
    SDL_assert(vertex_shader != NULL);
    SDL_assert(fragment_shader != NULL);

    SDL_GPUGraphicsPipeline* pipeline = SDL_CreateGPUGraphicsPipeline(device,
        &(SDL_GPUGraphicsPipelineCreateInfo){
            .target_info = {
		.num_color_targets = 1,
		.color_target_descriptions = (SDL_GPUColorTargetDescription[]){
                    {
			.format = SDL_GetGPUSwapchainTextureFormat(device, window),
		    },
                },
	    },
	    .vertex_input_state = (SDL_GPUVertexInputState){
		.num_vertex_buffers = 1,
		.vertex_buffer_descriptions = (SDL_GPUVertexBufferDescription[]){
                    {
		        .slot = 0,
			.input_rate = SDL_GPU_VERTEXINPUTRATE_VERTEX,
			.instance_step_rate = 0,
			.pitch = sizeof(Vertex),
		    },
                },
		.num_vertex_attributes = 2,
		.vertex_attributes = (SDL_GPUVertexAttribute[]){
                    {
		        .buffer_slot = 0,
			.format = SDL_GPU_VERTEXELEMENTFORMAT_FLOAT3,
			.location = 0,
			.offset = offsetof(Vertex, pos),
		    },
                    {
                        .buffer_slot = 0,
                        .format = SDL_GPU_VERTEXELEMENTFORMAT_FLOAT2,
                        .location = 1,
                        .offset = offsetof(Vertex, texcoords),
                    }
                }
	    },
	    .primitive_type = SDL_GPU_PRIMITIVETYPE_TRIANGLELIST,
	    .vertex_shader = vertex_shader,
	    .fragment_shader = fragment_shader,
        });
    LogAssertSDL(pipeline != NULL, "Failed to create pipeline");

    return pipeline;
}

void GraphicsPipelineBind(SDL_GPURenderPass* render_pass, SDL_GPUGraphicsPipeline* pipeline) {
    SDL_BindGPUGraphicsPipeline(render_pass, pipeline);
}

void GraphicsPipelineFree(SDL_GPUDevice* device, SDL_GPUGraphicsPipeline* pipeline) {
    SDL_assert(device != NULL);
    SDL_assert(pipeline != NULL);

    SDL_ReleaseGPUGraphicsPipeline(device, pipeline);
}
