#include <pch.h>

SDL_GPUShader* ShaderLoad(SDL_GPUDevice* device, const char* filename, SDL_GPUShaderStage stage,
    uint32_t num_uniform_buffers, uint32_t num_samplers) {
    SDL_assert(device != NULL);

    FileData file = FileReadBinary(filename);
    LogAssertErrnof(file.data != NULL, "Failed to read shader file: %s", filename);

    SDL_GPUShader* shader = SDL_CreateGPUShader(device,
        &(SDL_GPUShaderCreateInfo){
            .code = file.data,
            .code_size = file.length,
            .stage = stage,
            .format = SDL_GPU_SHADERFORMAT_SPIRV,
            .entrypoint = "main",
            .num_uniform_buffers = num_uniform_buffers,
            .num_samplers = num_samplers,
        });

    FileFree(file);

    LogAssertSDL(shader != NULL, "Failed to create shader");

    return shader;
}

void ShaderFree(SDL_GPUDevice* device, SDL_GPUShader* shader) {
    SDL_assert(device != NULL);
    SDL_assert(shader != NULL);

    SDL_ReleaseGPUShader(device, shader);
}
