#include <pch.h>

SDL_GPUTransferBuffer* TransferBufferCreate(
    SDL_GPUDevice* device, SDL_GPUTransferBufferUsage usage, size_t size) {
    SDL_assert(device != NULL);

    SDL_GPUTransferBuffer* transfer_buffer = SDL_CreateGPUTransferBuffer(device,
        &(SDL_GPUTransferBufferCreateInfo){
            .usage = usage,
            .size = size,
        });
    LogAssertSDL(transfer_buffer != NULL, "Failed to create GPU transfer buffer");

    return transfer_buffer;
}

void* TransferBufferMap(SDL_GPUDevice* device, SDL_GPUTransferBuffer* transfer_buffer) {
    void* mapped_buffer = SDL_MapGPUTransferBuffer(device, transfer_buffer, false);
    LogAssertSDL(mapped_buffer != NULL, "Failed to map GPU transfer buffer");

    return mapped_buffer;
}

void TransferBufferUnmap(SDL_GPUDevice* device, SDL_GPUTransferBuffer* transfer_buffer) {
    SDL_UnmapGPUTransferBuffer(device, transfer_buffer);
}

void TransferBufferFree(SDL_GPUDevice* device, SDL_GPUTransferBuffer* transfer_buffer) {
    SDL_assert(device != NULL);
    SDL_assert(transfer_buffer != NULL);

    SDL_ReleaseGPUTransferBuffer(device, transfer_buffer);
}
