#include <pch.h>

UploadState UploadBegin(SDL_GPUDevice* device, const void* data, size_t byte_size) {
    UploadState state = { 0 };

    state.transfer_buffer = TransferBufferCreate(device, SDL_GPU_TRANSFERBUFFERUSAGE_UPLOAD, byte_size);
    void* mapped_buffer = TransferBufferMap(device, state.transfer_buffer);
    memcpy(mapped_buffer, data, byte_size);
    TransferBufferUnmap(device, state.transfer_buffer);

    state.command_buffer = CommandBufferAcquire(device);
    state.copy_pass = CopyPassBegin(state.command_buffer);

    return state;
}

void UploadEnd(SDL_GPUDevice* device, const UploadState* state) {
    CopyPassEnd(state->copy_pass);
    CommandBufferSubmit(state->command_buffer);
    TransferBufferFree(device, state->transfer_buffer);
}

void UploadToBuffer(SDL_GPUDevice* device, const void* data, size_t byte_size, SDL_GPUBuffer* buffer) {
    SDL_assert(device != NULL);
    SDL_assert(data != NULL);
    SDL_assert(byte_size > 0);
    SDL_assert(buffer != NULL);

    UploadState upload_state = UploadBegin(device, data, byte_size);

    SDL_UploadToGPUBuffer(upload_state.copy_pass,
        &(SDL_GPUTransferBufferLocation){
            .transfer_buffer = upload_state.transfer_buffer,
            .offset = 0,
        },
        &(SDL_GPUBufferRegion){
            .buffer = buffer,
            .offset = 0,
            .size = byte_size,
        },
        false);

    UploadEnd(device, &upload_state);
}

void UploadToTexture(SDL_GPUDevice* device, SDL_Surface* surface, SDL_GPUTexture* texture) {
    UploadToTextureRaw(device, surface->pixels, surface->w, surface->h, texture);
}

void UploadToTextureRaw(
    SDL_GPUDevice* device, const void* pixels, int width, int height, SDL_GPUTexture* texture) {
    SDL_assert(device != NULL);
    SDL_assert(pixels != NULL);
    SDL_assert(width > 0);
    SDL_assert(height > 0);
    SDL_assert(texture != NULL);

    UploadState upload_state = UploadBegin(device, pixels, width * height * 4);

    SDL_UploadToGPUTexture(upload_state.copy_pass,
        &(SDL_GPUTextureTransferInfo){
            .transfer_buffer = upload_state.transfer_buffer,
            .offset = 0,
        },
        &(SDL_GPUTextureRegion){
            .texture = texture,
            .w = width,
            .h = height,
            .d = 1,
        },
        false);

    UploadEnd(device, &upload_state);
}
