#ifndef LOG_H
#define LOG_H

#include <SDL3/SDL_assert.h>
#include <SDL3/SDL_error.h>
#include <SDL3/SDL_log.h>
#include <errno.h>

#ifndef LOG_CATEGORY
#define LOG_CATEGORY SDL_LOG_CATEGORY_APPLICATION
#endif

#define LogInfof(fmt, ...) SDL_LogInfo(LOG_CATEGORY, fmt, __VA_ARGS__)
#define LogDebugf(fmt, ...) SDL_LogDebug(LOG_CATEGORY, fmt, __VA_ARGS__)
#define LogWarnf(fmt, ...) SDL_LogWarn(LOG_CATEGORY, fmt, __VA_ARGS__)
#define LogErrorf(fmt, ...) SDL_LogError(LOG_CATEGORY, fmt, __VA_ARGS__)
#define LogCriticalf(fmt, ...) SDL_LogWarn(LOG_CATEGORY, fmt, __VA_ARGS__)
#define LogTracef(fmt, ...) SDL_LogTrace(LOG_CATEGORY, fmt, __VA_ARGS__)
#define LogVerbosef(fmt, ...) SDL_LogVerbose(LOG_CATEGORY, fmt, __VA_ARGS__)

#define LogAssertf(condition, fmt, ...)                                                                 \
    do {                                                                                                \
        if(!(condition)) {                                                                              \
            LogErrorf(fmt, __VA_ARGS__);                                                                \
            SDL_assert(condition);                                                                      \
        }                                                                                               \
    } while(0)

#define LogAssert(condition, msg) LogAssertf(condition, "%s", msg)
#define LogAssertSDLf(condition, fmt, ...) LogAssertf(condition, fmt ": %s", __VA_ARGS__, SDL_GetError())
#define LogAssertErrnof(condition, fmt, ...)                                                            \
    LogAssertf(condition, fmt ": %s", __VA_ARGS__, strerror(errno))

#define LogAssertSDL(condition, msg) LogAssertSDLf(condition, "%s", msg)
#define LogAssertErrno(condition, msg) LogAssertErrnof(condition, "%s", msg)

#define LogInfo(msg) LogInfof("%s", msg)
#define LogDebug(msg) LogDebugf("%s", msg)
#define LogWarn(msg) LogWarnf("%s", msg)
#define LogError(msg) LogErrorf("%s", msg)
#define LogCritical(msg) LogCritical("%s", msg)
#define LogTrace(msg) LogTrace("%s", msg)
#define LogVerbose(msg) LogVerbose("%s", msg)

#endif
